-----------------------------------------------------------------------------------------------
-- Client Lua Script for CrashCourse
-- Copyright (c) Alex Martin. Released under MIT license. 
-----------------------------------------------------------------------------------------------
 
require "Window"
require "GroupLib"
 
-----------------------------------------------------------------------------------------------
-- CrashCourse Module Definition
-----------------------------------------------------------------------------------------------
local CrashCourse = {}
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------

-----------------------------------------------------------------------------------------------
-- "Globals" (within the addon context)
-----------------------------------------------------------------------------------------------

local diCrashCourses = {} -- a dictionary of {"mob_name":["Message 1", "Message 2"...], ...}
local arSilencedMobs = {} -- an array of mobs that we have already said "crash course available for mob" once this session.
local strCurrentTarget = ""
local strSlashCommand = "crash"
local debug = false
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function CrashCourse:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 
    return o
end

function CrashCourse:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- CrashCourse OnLoad
-----------------------------------------------------------------------------------------------
function CrashCourse:OnLoad()
	Apollo.RegisterSlashCommand(strSlashCommand, "OnCrashCourseOn", self)
	--Apollo.RegisterSlashCommand("getzone", "OnCrashGetZone", self)
	--Apollo.RegisterEventHandler("VarChange_ZoneName", "OnChangeZoneName", self)
	Apollo.RegisterEventHandler("TargetUnitChanged", "OnTargetUnitChanged", self)
	
	-- init rover
	self:PublishVarsToRover()
	
	-- load mobs
	self:LoadMobs()
end

function CrashCourse:PublishVarsToRover()
	if debug then
		SendVarToRover("CC: diCrashCourses", diCrashCourses)
		SendVarToRover("CC: arSupportedZones", arSupportedZones)
		SendVarToRover("CC: arSilencedMobs", arSilencedMobs)
		SendVarToRover("CC: strCurrentTarget", strCurrentTarget)
		SendVarToRover("CC: bCurrentZoneSupported", bCurrentZoneSupported)
		SendVarToRover("CC: strCurrentZone", strCurrentZone)
	end
end

-----------------------------------------------------------------------------------------------
-- CrashCourse Event Handlers
-----------------------------------------------------------------------------------------------

function CrashCourse:OnTargetUnitChanged(unitTarget)
	if unitTarget == nil then
		strCurrentTarget = ""
		return
	end
	
	strCurrentTarget = unitTarget:GetName()
	if self:MobIsSupported(strCurrentTarget) == true and self:MobOutputIsSilenced(strCurrentTarget) == false then
		table.insert(arSilencedMobs, self:JSONifyString(strCurrentTarget))
		self:WriteLog("Type '/" .. strSlashCommand .. "' to output target")
	end
	
	self:PublishVarsToRover()
end

-----------------------------------------------------------------------------------------------
-- CrashCourse Functions
-----------------------------------------------------------------------------------------------

-- on SlashCommand "/cc"
function CrashCourse:OnCrashCourseOn(cmd, args)
	local strTargetJSON = ""
	
	-- get the mob that we need to output
	if args == "" then
		self:WriteDebug("Showing crash course for target")
	else
		self:WriteDebug("Showing crash course for mob: " .. args)
		strCurrentTarget = args
	end
	
	-- get the messages to print
	strTargetJSON = self:JSONifyString(strCurrentTarget)
	self:WriteDebug("Searching for mob: " .. strTargetJSON)
	local arMobMessages = diCrashCourses[strTargetJSON]
	
	-- print the messages
	if arMobMessages ~= nil then
		self:OutputToGroup("[CrashCourse] -- Strategy for " .. strCurrentTarget .. " --")
		for _,msg in pairs(arMobMessages) do
			self:OutputToGroup(msg)
		end
	else
		self:WriteLog("No crash course for this mob")
	end
	
	self:PublishVarsToRover()
end

function CrashCourse:MobOutputIsSilenced(strMobName)
	local strMobJSON = self:JSONifyString(strMobName)
	
	-- look through the silenced mobs array
	for _,val in pairs(arSilencedMobs) do	
		-- if the mob is there, return true
 		if val == strMobJSON then
    		return true
    	end
 	end

	-- if we get here, return false
	return false
end

function CrashCourse:OutputToGroup(strText)
	if GroupLib.InInstance() == true then
		ChatSystemLib.Command(string.format("/i %s", strText))
	elseif GroupLib.InGroup() == true then
		ChatSystemLib.Command(string.format("/p %s", strText))
	else
		self:WriteLog(strText)
	end
end

function CrashCourse:WriteLog(strText)
	ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, strText, "CrashCourse")
end

function CrashCourse:WriteDebug(strText)
	if debug then
		ChatSystemLib.PostOnChannel(ChatSystemLib.ChatChannel_System, strText, "CC Debug")
	end
end

function CrashCourse:MobIsSupported(strMobName)
	local strMobJSON = self:JSONifyString(strMobName)
	
	-- look through the mobs dictionary
	for key,val in pairs(diCrashCourses) do
	
		-- if the zone is in the array, return true
 		if key == strMobJSON then
    		return true
    	end
 	end

	-- if we get here, return false
	return false
end

function CrashCourse:JSONifyString(strOriginal)
	-- equivalent to var edited = str.toLowerCase().replace(/[\s]/g, '_').replace(/[^_a-z0-9]/gi, '');
	local strNew = strOriginal:lower()
	strNew = strNew:gsub("%s", "_")
	strNew = strNew:gsub("[^_a-zA-Z0-9]", "")
	return strNew
end

function CrashCourse:SpellDesc(strName, bInterrupt, iIntArmor, bOptional, bVeteran)
	-- defaults
	iIntArmor = iIntArmor or 0
	bOptional = bOptional or false
	bInterrupt = bInterrupt or false
	bVeteran = bVeteran or false
	
	-- build the string
	local strDesc = strName
	local strInterrupt = "**Interrupt** "
	local strDodge = "Dodge out of "
	local strIntArmor = "(" .. iIntArmor .. " IA)"
	local strIntArmorInfinite = "(∞ IA)"
	local strOptional = "(Optional)"
	local strVeteran = "Veteran: "
	
	if iIntArmor > 0 then
		strDesc = strDesc .. " " .. strIntArmor
	end
	
	if iIntArmor == -1 then
		strDesc = strDesc .. " " .. strIntArmorInfinite
	end
	
	if bOptional == true then
		strDesc = strDesc .. " " .. strOptional
	end
	
	if bInterrupt then
		strDesc = strInterrupt .. strDesc
	else
		strDesc = strDodge .. strDesc
	end
	
	if bVeteran then
		strDesc = strVeteran .. strDesc
	end
	
	return strDesc
end

-----------------------------------------------------------------------------------------------
-- Initialize the "supported mobs" array (it's down here because it's long)
-----------------------------------------------------------------------------------------------

function CrashCourse:LoadMobs()
	local mobs = {}
	
	-- Testing
	mobs["trade_management_facilitator"] = {
		"A very dangerous opponent for your wallet!",
		self:SpellDesc("CREDD!", false, 999, true)
	}
	
	-- Kel Vorath
	mobs["blood_pit_gladiator"] = {
		self:SpellDesc("Mighty Swing"),
		self:SpellDesc("Gladiator Rampage (3 jumps total)", false, -1),
		"The four adds close by will pull."
	}
	mobs["blood_pit_necromancer"] = {
		self:SpellDesc("Wail of the Darkwitch (will expand 3 times)", true),
		"Will summon 3 Blood Pit Skeletons. Ignore them."
	}
	mobs["grond_the_corpsemaker"] = {
		self:SpellDesc("Thrash", true, 2),
		self:SpellDesc("Bellow"),
		self:SpellDesc("Mutilate", false, 0, false),
		"Tank Grond in the center of the room.",
		"Stack on Grond's right shoulder (front and rear cleave).",
		"Grond will randomly knock the tank back.",
		"Take a CC break to get out of Mutilate!",
		"Kite adds to bone traps ASAP, avoid them yourself."
	}
	mobs["enslaved_protector"] = {
		self:SpellDesc("Exterminate", true, 1),
		"Ignore adds"
	}
	mobs["enslaved_augmentor"] = {
		self:SpellDesc("Rejuvinate", true, 1)
	}
	mobs["voreth_mechano_slaver"] = {
		self:SpellDesc("Rejuvinate", true, 1) .. " on Enslaved Augmentor"
	}
	mobs["voreth_necromancer"] = {
		self:SpellDesc("Wail of the Darkwitch", true, 1),
		self:SpellDesc("Raise Dead", true, 1, true)
	}
	mobs["darkwitch_gurka"] = {
		self:SpellDesc("Afflicted Soul"),
		self:SpellDesc("Binding Dark", true, 2)
	}
	mobs["slavemaster_drokk"] = {
		self:SpellDesc("Suppression Wave", true, 2, true),
		"When Drokk goes behind the green curtain, stack on the curtain and kill the Bombshell Construct that is right there- avoid the rest",
		"When tethered, kill your tether (or break out via a skill), then assist your healer -> tank -> other DPS",
		"When Destructo Constructs spawn, if you are the target run away. Otherwise assist with CC or killing constructs that get too close to the target",
		"Sub 20%, Drokk will summon Bombshell Constructs while fighting. Move the boss out of the blast radius"
	}
	mobs["voreth_beastmaster"] = {
		"Ignore the dogs",
		self:SpellDesc("Pulverize", true, 2)
	}
	mobs["voreth_beastmaster"] = {
		"Ignore the dogs",
		self:SpellDesc("Pulverize", true, 2)
	}
	mobs["voreth_blinded_slaver"] = {
		"Ignore adds",
		"Stand behind to avoid telegraphs"
	}
	mobs["voreth_darkwitch"] = {
		self:SpellDesc("Rupture", true, 2),
		self:SpellDesc("Impending Doom", true, 2, true),
		"Stack for healing and dispels",
		"Dispel the DoT- it hits hard"
	}
	mobs["voreth_battlesworn"] = {
		self:SpellDesc("Hammer of Wrath", true, 2),
		self:SpellDesc("Seismic Trample", true, 2, true)
	}
	mobs["forgemaster_trogun"] = {
		"Attempt to stack for healing",
		"When Trogun begins channeling Forgemaster's Call, intercept the balls of fire heading towards him. Balls of fire will give a damage and support power buff to players",
		"Attempt to keep the boss away from the AOEs he leaves on the floor, but as close to the center of the room as possible",
		"When Trogun begins casting Exanite Weapon, stand at the opposite side of the room and dodge the projectiles that begin radiating from him",
		"After casting Exanite Weapon, Trogun will radiate projectiles while casting Forgemaster's Call. Dodge these while intercepting the balls of fire"
	}
	
	-- stormtalon's lair
	mobs["thundercall_sentinel"] = {
		self:SpellDesc("Shockwave", true)
	}
	mobs["thundercall_shaman"] = {
		self:SpellDesc("Rejuvinate", true, 1)
	}
	mobs["thundercall_stormwatcher"] = {
		self:SpellDesc("Rampage", true, 1),
		self:SpellDesc("Hack and Slash")
	}
	mobs["thundercall_stormweaver"] = {
		"THE MOST DANGEROUS TRASH in this dungeon",
		"Applies a debuff on your group, which will stack as long as he remains alive. The debuff will do periodic damage. Needs to be Cleansed",
		self:SpellDesc("Conduction", true, 1)
	}
	mobs["bladewind_the_invoker"] = {
		self:SpellDesc("Thunder Cross", true, 2),
		"At 60% the boss will disarm the group and become immune. Grab your weapon and kill the Thundercall Channelers in a clockwise direction.",
		"If possible, drop your Lightning Strike telegraph on the Thundercall Channeler you are attacking",
		"Once all four channelers are dead, the boss will disarm the group. Tank, ensure you get your weapon ASAP to taunt.",
		"Avoid wisps (they will stun) and kill the boss."
	}
	mobs["aethros"] = {
		"Phase 1:",
		"Boss will only hit the tank, nothing special.",
		"When the boss summons the three adds, cycle CC on the adds and kill them ASAP.",
		"Phase 2:",
		"Run to the boss dodging tornados and telegraphs. When enough people are close, interrupt the boss's spell (2 interrupt armor).",
		"Watch for large telegraphs around the boss after interrupting, dodge these.",
		"After interrupting, the fight will switch back to Phase 1. Continue these until the fight ends."
	}
	mobs["arcanist_breezebinder"] = {
		self:SpellDesc("Gust Convergence", true, 2),
		"Gust Convergence casts quickly and is VERY high priority- if not interrupted you will likely wipe.",
		self:SpellDesc("Manifest Cyclone", false)
	}
	mobs["stormtalon"] = {
		"All but tank stack on the side of the boss as he does a frontal and rear cleave",
		"Players will be chosen at random for continual small telegraphs cast on them- run until they stop and don't drop them on the group",
		"After you get knocked back, get out of the stun and interrupt the boss ASAP",
		"Once under 60% HP the boss will sometimes choose a random player and make them the 'eye of the storm'. Everyone else needs to stack on this player.",
		"The player who is the eye of the storm needs to move slowly to the side to avoid telegraphs.",
		"Ensure that the eye of the storm touches the boss so that melee DPS can continue to deal damage."
	}
	
	-- Skullcano
	mobs["grimgrim_blightmask"] = {
		"Blightbrush: telegraphs will appear, if you get hit by them you'll get stunned and take damage. This can be interrupted."
	}
	mobs["ravenous_razorbeak"] = {
		"Ravenous Rush: frontal telegraph, if you get hit by it you'll get knocked down.",
		"Berserk: damage increase"
	}
	mobs['grimgrim_hexweaver'] = {
		'Essence Drain: he will drain someones HP, this can be interrupted, but if you have enough dps you should deal him more damage than his regen.'
	}
	mobs['grimgrim_doomcaller'] = {
		self:SpellDesc('Condemnation', true, 2)
	}
	mobs['redmoon_jailer'] = {
		'Redmoon Prisoners sometimes choose random people to aggro',
		'Focus the jailer as once he is dead the prisoners will run away'
	}
	mobs['redmoon_gold_guardian'] = {
		self:SpellDesc('Marauder Cleave', true, 1)
	}
	mobs['redmoon_gold_hoarder'] = {
		self:SpellDesc('Debilitating Barrage', true)
	}
	mobs['redmoon_canoneer'] = {
		self:SpellDesc('Marauder Mortar', true, 1)
	}
	mobs['redmoon_razortail'] = {
		self:SpellDesc('Clawing Frenzy', true)
	}
	mobs['redmoon_wrestler'] = {
		'Fixation: Once the Wrestler is done casting it, he will focus his actual target and try to reach him.',
		'Trample: after fixation is done casting, a wide frontal telegraph will appear before the Wrestler, if this telegraph reaches the target of Fixation(which will usually always be the tank) the Wrestler will take him for a little walk in the air like a WWE wrestler. Wrestlers always come by pairs, and will usually always cast Fixation at the same time. Thus knocking their target in the air twice in a row. This does does medium damage even twice in a row. It also interrupts the player.'
	}
	mobs['redmoon_crusher'] = {
		"Leap: he will leap on a targeted area and disorient any person standing inside it. This is an instant skill that can't be interrupted and can hardly be avoided.",
		'Twinkle Toes: frontal telegraph that can be avoided or interrupted.'
	}
	mobs['redmoon_furyspitter'] = {
		"Gold swell: This will create telegraphs that will fill with gold once the cast is over. Standing or stepping on them once the cast is finished will stun you. These These telegraphs stay on the ground until the Furyspitter is killed",
		"He will also randomly make a missile fall on a targeted zone."
	}
	mobs['redmoon_necromancer'] = {
		self:SpellDesc('Vitae Siphon', true, 2)
	}
	mobs['redmoon_sniper'] = {
		self:SpellDesc('Dead Eye', true)
	}
	mobs['stewshaman_tugga'] = {
		"He is fairly simple and doesn't do a lot of damage. He will rotate between different skills:",
		"Molten Rain: Molten Lava will rain down on you, just avoid the telegraphs. The telegraphs take a lot of time to tick and can be easily avoided. You don't need to interrupt this.",
		"Feeding Frenzy Phase: The following message will be displayed 'Stew-Shaman enters a feeding frenzy!' this will be the first test for your group. This will knock down and disarm everyone. Meat will appear all around the cauldron. The boss will try to reach for it and eat it. If you allow him to do so he will regenerate a crazy amount of HP. While he eats he will cast 'Devour Flesh'(this just means that he's eating). You can interrupt him while he eats. And you should interrupt him each time he eats. Your group needs to eat the meat before he has the chance to do so in order to keep him from regenerating. Allowing him to eat meat will not only regenerate him, it will also give him a damage buff.",
		"During this feeding frenzy there will be no damage at all so the healer should be eating meat too. Basically you are going to need the tank, the healer and 1 dps to eat meat while the other 2 dps continue doing damage and interrupting him if he tries to eat meat.",
		"Alternately, experienced groups may stun the boss while he is eating to take advantage of chained MoOs."
	}
	mobs['thunderfoot'] = {
		"That's a Big Bad Gorilla who instead of throwing bananas likes punching people instead. Lucky for us it turns out he is allergic to mushrooms, so let's give him some shall we?",
		"This boss can't be interrupted.",
		"Thunder Pound: he will pound the ground in front of him. This can't be interrupted and needs to be avoided. In order to avoid this you will need to outside the telegraph but also to jump when he hits the ground with his fist in order to avoid the aoe damage from it.",
		"Enraging Fury: damage buff",
		"Pulverize: this will make a big telegraph appear around him. He will jump and pound the ground with his feet. You need to get out of this telegraph and jump when he touches the ground in order to avoid the aoe damage from this attack.",
		"His normal attack is a cleave so no one except the tank should stay in front of him. During this fight the following message will appear 'A colossal mushroom has sprouted'. Colossal Mushrooms will sprout, in other words a BIG mushroom will appear in the room. The tank will need to take the boss near that mushroom in order for the boss to hit it with his cleave or one of his attacks. When the boss hits a colossal mushroom he gets stunned and goes into a MoO phase, allowing you to deal more damage to him. You need to repeat this action as soon as a Colossal Mushroom sprouts.",
		"During the fight Moodies outside the boss's room will also through stuff at you, just avoid their telegraphs."
	}
	mobs['bosun_octog'] = {
		"He has two phases that he will alternate between.",
		"Phase 1:",
		"Shred: he will bash the ground in front of him. Just avoid it.",
		"Hookshot: a targeted frontal telegraph that will grip to him anyone standing on it. The targeted player will always be gripped. Other players should avoid it.",
		"Bird Swoop: thick linear telegraphs will appear randomly through Phase 1 as Octog's bird companion swoops through. Just move out of them, if you stay on them you'll get knocked down.",
		"Phase 2:",
		"Ink Shield: this will make a shield appear around him. He will still take damage from attacks but it will be greatly reduced.",
		"Noxious Ink: During this phase you will get the following message '(Insert player name) is dripping Noxious Ink!'. The person targeted by this needs to keep running until the debuff is gone in order to not get damage from this. He will leave a trail of Noxious Ink behind him, so be careful as to where you run to.",
		"Squirg adds: Many squirg adds spawn during Ink Shield. These adds are bombs that will grow in size. They explode when they reach a certain size or hit a player. Avoid and/or kill them.",
		"Razooki: Octog's monkey companion usually spawns during Phase 2, but he can also spawn during Phase 1. He has very little HP and should be focused down as soon as he spawns."
	}
	mobs['quartermaster_gruh'] = {
		'Dead Eye: same as the sniper',
		self:SpellDesc('Kneecap', true)
	}
	mobs['mordechai_redmoon'] = {
		"Now that's one tough bastard that will most likely ruin a lot of gold runs during live.",
		"Phase 1:",
		"The Terraformer begins to rumble: this message will be displayed and will be the cue for the beginning of Phase 1.",
		"At pull he will disappear and 4 beams will appear around the room. Along with those beams will appear Gold Swells(same as the Furyspitter's ones). The beams will start moving in a direction, you need to avoid getting hit by them. 1 tick from them will deal 80% of your HP as damage. You need to stay ahead of those beams while avoiding the Gold Swells on the ground. You can jump over Gold Swells, you won't get stunned but you'll take a bit of damage.",
		"At some point beams will disappear and that's when Mordechai will appear again.",
		"Phase 1 usually happens 3 times during this fight. Every time it will happen the beams will move faster. In order to stay ahead of them you'll need to run near the center of the room, but avoid running too close to the center as you could fall since the edges are tricky and you could fall into the lava pit, this will be an insta death, and fail for the gold run.",
		"Phase 2:",
		"Auto Attack: his auto attack is a frontal and back linear telegraph. This means everyone except the tank needs to stay on the side of the boss to dps/heal.",
		"The Terraformer is overcharging : at some point the following message will be displayed 'The Terraformer is overcharging!'. When you see this just look away from the center of the room. In other words turn your back on the lava pit in the center of the room. Doing so will avoid you getting blinded by the Terraformer. Once the overcharging is done Mordechai will be under a MoO if at least one of your party members did not get blinded by the Terraformer.",
		"Turret Grenade: this will launch a turret to the targeted zone. These turrets do quite a lot of damage. You need to get out of range of them. It is crucial for the tank to move the boss away from them, and for the rest of the group to follow his lead.",
		"Big Bang: he will jump on someone, stun him, and begin casting this skill on him. Just spam F to get out of the stun and move away from the telegraph. The cast time is ridiculously long, this can be interrupted very easily.",
		"Aoe: He will also cast a big aoe with safe zones in it. This aoe will tick multiple times, each time it ticks the safe zones will move clockwise. Just interrupt this skill."
	}
	
	diCrashCourses = mobs
end

-----------------------------------------------------------------------------------------------
-- CrashCourse Instance
-----------------------------------------------------------------------------------------------
local CrashCourseInst = CrashCourse:new()
CrashCourseInst:Init()
